package com.css.simulation.resource.scheduler.service;

import com.css.simulation.resource.scheduler.configuration.redis.CustomRedisClient;
import com.css.simulation.resource.scheduler.entity.PrefixEntity;
import com.css.simulation.resource.scheduler.entity.ProjectEntity;
import com.css.simulation.resource.scheduler.entity.TaskEntity;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.util.ProjectUtil;
import com.css.simulation.resource.scheduler.util.TaskUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
@Slf4j
public class TaskService {
    @Resource
    private TaskUtil taskUtil;
    @Resource
    private TaskMapper taskMapper;
    @Resource
    private ProjectUtil projectUtil;
    @Resource
    private CustomRedisClient customRedisClient;


    // -------------------------------- Comment --------------------------------

    @SneakyThrows
    public void taskState(String taskId, String state, String podName) {
        TaskEntity taskEntity = taskMapper.selectById(taskId);
        String projectId = taskEntity.getPId(); // 项目 id
        String lock1 = "taskId:" + taskId + ":state:" + state + ":pod-name:" + podName;
        String lock2 = "project:" + projectId + ":completed-lock";
        customRedisClient.lock(lock1, 1L, 30 * 60L);
        try {
            // 查询相关信息
            ProjectEntity projectEntity = projectUtil.getProjectByProjectId(projectId);
            String projectType = projectEntity.getProjectType();  // 项目类型
            String maxSimulationTime = projectEntity.getMaxSimulationTime();  // 项目类型
            String userId = taskEntity.getCreateUserId();   // 用户 id
            PrefixEntity redisPrefix = projectUtil.getRedisPrefixByUserIdAndProjectIdAndTaskId(userId, projectId, taskId);
            // 判断是否完成
            boolean projectCompleted = taskUtil.isProjectCompleted(redisPrefix, projectId, projectType, maxSimulationTime, taskId, state, podName);
            if (projectCompleted) {
                customRedisClient.tryLock(lock2, 10 * 60L);
                log.info("项目 {} 开始打分。", projectId);
                taskUtil.score(redisPrefix.getProjectRunningKey(), userId, projectId, projectType);
                log.info("项目 {} 计算评价等级。", projectId);
                taskUtil.evaluationLevel(projectId);
                log.info("项目 {} 开始释放资源。", projectId);
                taskUtil.done(redisPrefix, projectId, projectType);
                log.info("项目 {} 运行结束。", projectId);
            }
        } finally {
            customRedisClient.unlock(lock2);
            customRedisClient.unlock(lock1);
        }

    }


    public Boolean taskConfirm(String taskId) {
        return taskUtil.taskConfirm(taskId);
    }

    public void taskTick(String taskId) {
        taskUtil.taskTick(taskId);
    }


}



